import os
import geoip2.database
import geoip2.errors
import logging

logger = logging.getLogger(__name__)


class GeoLite:
    path = os.path.dirname(os.path.abspath(__file__))
    reader_info = geoip2.database.Reader(path + '/GeoLite2-City.mmdb', locales='zh-CN')
    reader_isp = geoip2.database.Reader(path + '/GeoLite2-ASN.mmdb', locales='zh-CN')

    @classmethod
    def get_info_by_ip(cls, ip):
        info = {
            "city": "",  # 城市
            "continent": "",  # 大洲
            "country": "",  # 国家
            # "location": {
            "lat": "",
            "lon": "",
            # },
            # "registered_country": "",
            "subdivisions": "",
            "isp": "",
            'asn': ""
        }
        try:
            res = cls.reader_info.city(ip)
            info['asn'] = cls.get_asn_by_ip(ip)
            info['isp'] = cls.get_isp_by_ip(ip)
            info['lat'] = res.location.latitude
            info['lon'] = res.location.longitude
            info['continent'] = res.continent.names[res._locales] if res.continent.names else ''
            info['country'] = res.country.names[res._locales] if res.country.names else ''
            info['city'] = res.city.names[res._locales] if res.city.names and 'zh-CN' in res.city.names else ''
            if res.subdivisions:
                try:
                    info['subdivisions'] = res.subdivisions[0].names[res._locales]
                except:
                    info['subdivisions'] = res.subdivisions[0].names['en']
            else:
                info['subdivisions'] = ''

            info['addr'] = info['country']
            if info['subdivisions']:
                info['addr'] += '-' + info['subdivisions']
            if info['city'] and info['city'] != info['subdivisions']:
                info['addr'] += '-' + info['city']

            if ip == '127.0.0.1':
                info['addr'] = '本机地址'
            elif ip.startswith('127.'):
                info['addr'] = '局域网'
            elif ip.startswith('192.168.'):
                info['addr'] = '局域网'
            return info
        except geoip2.errors.AddressNotFoundError as e:
            if ip == '127.0.0.1':
                info['addr'] = '本机地址'
            elif ip.startswith('127.'):
                info['addr'] = '局域网'
            elif ip.startswith('192.168.'):
                info['addr'] = '局域网'
            else:
                info['addr'] = '未知'
        except Exception as e:
            logger.warning(e, exc_info=True)
            if ip == '127.0.0.1':
                info['addr'] = '本机地址'
            elif ip.startswith('127.'):
                info['addr'] = '局域网'
            elif ip.startswith('192.168.'):
                info['addr'] = '局域网'
            else:
                info['addr'] = '未知'
        return info

    @classmethod
    def get_isp_by_ip(cls, ip):
        return cls.reader_isp.asn(ip).autonomous_system_organization

    @classmethod
    def get_asn_by_ip(cls, ip):
        return cls.reader_isp.asn(ip).autonomous_system_number


if __name__ == "__main__":
    GeoLite.get_info_by_ip("95.210.119.104")



# 纯真数据库
import socket
import struct
import os


class CzIp:
    db_path = os.path.dirname(os.path.abspath(__file__)) + '/qqwry.dat'

    def __init__(self, db_file=''):
        if not db_file:
            db_file = self.db_path
        self.f_db = open(db_file, "rb")
        bs = self.f_db.read(8)
        (self.first_index, self.last_index) = struct.unpack('II', bs)
        self.index_count = int((self.last_index - self.first_index) / 7 + 1)
        self.cur_start_ip = None
        self.cur_end_ip_offset = None
        self.cur_end_ip = None
        # print(self.get_version(), " 纪录总数: %d 条 "%(self.index_count))

    def get_version(self):
        '''
        获取版本信息，最后一条IP记录 255.255.255.0-255.255.255.255 是版本信息
        :return: str
        '''
        s = self.get_addr_by_ip(0xffffff00)
        # print(s)
        return s

    def _get_area_addr(self, offset=0):
        if offset:
            self.f_db.seek(offset)
        bs = self.f_db.read(1)
        (byte,) = struct.unpack('B', bs)
        if byte == 0x01 or byte == 0x02:
            p = self.getLong3()
            if p:
                return self.get_offset_string(p)
            else:
                return ""
        else:
            self.f_db.seek(-1, 1)
            return self.get_offset_string(offset)

    def _get_addr(self, offset):
        '''
        获取offset处记录区地址信息(包含国家和地区)
        如果是中国ip，则是 "xx省xx市 xxxxx地区" 这样的形式
        (比如:"福建省 电信", "澳大利亚 墨尔本Goldenit有限公司")
        :param offset:
        :return:str
        '''
        self.f_db.seek(offset + 4)
        bs = self.f_db.read(1)
        (byte,) = struct.unpack('B', bs)
        if byte == 0x01:  # 重定向模式1
            country_offset = self.getLong3()
            self.f_db.seek(country_offset)
            bs = self.f_db.read(1)
            (b,) = struct.unpack('B', bs)
            if b == 0x02:
                country_addr = self.get_offset_string(self.getLong3())
                self.f_db.seek(country_offset + 4)
            else:
                country_addr = self.get_offset_string(country_offset)
            area_addr = self._get_area_addr()
        elif byte == 0x02:  # 重定向模式2
            country_addr = self.get_offset_string(self.getLong3())
            area_addr = self._get_area_addr(offset + 8)
        else:  # 字符串模式
            country_addr = self.get_offset_string(offset + 4)
            area_addr = self._get_area_addr()
        return country_addr + " " + area_addr

    def dump(self, first, last):
        '''
        打印数据库中索引为first到索引为last(不包含last)的记录
        :param first:
        :param last:
        :return:
        '''
        if last > self.index_count:
            last = self.index_count
        for index in range(first, last):
            offset = self.first_index + index * 7
            self.f_db.seek(offset)
            buf = self.f_db.read(7)
            (ip, of1, of2) = struct.unpack("IHB", buf)
            address = self._get_addr(of1 + (of2 << 16))
            print("%d %s %s" % (index, self.ip2str(ip), address))

    def _set_ip_range(self, index):
        offset = self.first_index + index * 7
        self.f_db.seek(offset)
        buf = self.f_db.read(7)
        (self.cur_start_ip, of1, of2) = struct.unpack("IHB", buf)
        self.cur_end_ip_offset = of1 + (of2 << 16)
        self.f_db.seek(self.cur_end_ip_offset)
        buf = self.f_db.read(4)
        (self.cur_end_ip,) = struct.unpack("I", buf)

    def get_addr_by_ip(self, ip):
        '''
        通过ip查找其地址
        :param ip: (int or str)
        :return: str
        '''
        if type(ip) == str:
            ip = self.str2ip(ip)
        L = 0
        R = self.index_count - 1
        while L < R - 1:
            M = int((L + R) / 2)
            self._set_ip_range(M)
            if ip == self.cur_start_ip:
                L = M
                break
            if ip > self.cur_start_ip:
                L = M
            else:
                R = M
        self._set_ip_range(L)
        # version information, 255.255.255.X, urgy but useful
        if ip & 0xffffff00 == 0xffffff00:
            self._set_ip_range(R)
        if self.cur_start_ip <= ip <= self.cur_end_ip:
            address = self._get_addr(self.cur_end_ip_offset)
        else:
            address = ''
        return address

    def get_ip_range(self, ip):
        '''
        返回ip所在记录的IP段
        :param ip: ip(str or int)
        :return: str
        '''
        if type(ip) == str:
            ip = self.str2ip(ip)
        self.get_addr_by_ip(ip)
        range = self.ip2str(self.cur_start_ip) + ' - ' \
                + self.ip2str(self.cur_end_ip)
        return range

    def get_offset_string(self, offset=0):
        '''
        获取文件偏移处的字符串(以'\0'结尾)
        :param offset: 偏移
        :return: str
        '''
        if offset:
            self.f_db.seek(offset)
        bs = b''
        ch = self.f_db.read(1)
        (byte,) = struct.unpack('B', ch)
        while byte != 0:
            bs += ch
            ch = self.f_db.read(1)
            (byte,) = struct.unpack('B', ch)
        return bs.decode('gbk')

    def ip2str(self, ip):
        '''
        整数IP转化为IP字符串
        :param ip:
        :return:
        '''
        return str(ip >> 24) + '.' + str((ip >> 16) & 0xff) + '.' + str((ip >> 8) & 0xff) + '.' + str(ip & 0xff)

    def str2ip(self, s):
        '''
        IP字符串转换为整数IP
        :param s:
        :return:
        '''
        (ip,) = struct.unpack('I', socket.inet_aton(s))
        return ((ip >> 24) & 0xff) | ((ip & 0xff) << 24) | ((ip >> 8) & 0xff00) | ((ip & 0xff00) << 8)

    def getLong3(self, offset=0):
        '''
        3字节的数值
        :param offset:
        :return:
        '''
        if offset:
            self.f_db.seek(offset)
        bs = self.f_db.read(3)
        (a, b) = struct.unpack('HB', bs)
        return (b << 16) + a



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time    : 2020/3/31 10:51 AM
# @Author  : w8ay
# @File    : enums.py

# We define some constants
class DBMS:
    DB2 = 'IBM DB2 database'
    MSSQL = 'Microsoft SQL database'
    ORACLE = 'Oracle database'
    SYBASE = 'Sybase database'
    POSTGRE = 'PostgreSQL database'
    MYSQL = 'MySQL database'
    JAVA = 'Java connector'
    ACCESS = 'Microsoft Access database'
    INFORMIX = 'Informix database'
    INTERBASE = 'Interbase database'
    DMLDATABASE = 'DML Language database'
    SQLITE = 'SQLite database'
    UNKNOWN = 'Unknown database'


class OS(object):
    LINUX = "Linux"
    WINDOWS = "Windows"
    DARWIN = "Darwin"


class PLACE:
    GET = "GET"
    POST = "POST"
    URI = "URI"
    COOKIE = "Cookie"
    USER_AGENT = "User-Agent"
    REFERER = "Referer"
    HOST = "Host"


class HTTPMETHOD(object):
    GET = "GET"
    POST = "POST"
    HEAD = "HEAD"
    PUT = "PUT"
    DELETE = "DELETE"
    TRACE = "TRACE"
    OPTIONS = "OPTIONS"
    CONNECT = "CONNECT"
    PATCH = "PATCH"


class POST_HINT(object):
    NORMAL = "NORMAL"
    SOAP = "SOAP"
    JSON = "JSON"
    JSON_LIKE = "JSON-like"
    MULTIPART = "MULTIPART"
    XML = "XML (generic)"
    ARRAY_LIKE = "Array-like"


class WEB_PLATFORM(object):
    PHP = "Php"
    ASP = "Asp"
    ASPX = "Aspx"
    JAVA = "Java"
    PYTHON = 'Python'


class Level(object):
    NONE = 0
    LOW = 1
    MIDDLE = 2
    HIGHT = 3


POST_HINT_CONTENT_TYPES = {
    POST_HINT.JSON: "application/json",
    POST_HINT.JSON_LIKE: "application/json",
    POST_HINT.MULTIPART: "multipart/form-data",
    POST_HINT.SOAP: "application/soap+xml",
    POST_HINT.XML: "application/xml",
    POST_HINT.ARRAY_LIKE: "application/x-www-form-urlencoded; charset=utf-8",
}


class VulType(object):
    CMD_INNJECTION = "cmd_injection"
    CODE_INJECTION = "code_injection"
    XSS = "xss"
    SQLI = "sqli"
    DIRSCAN = "dirscan"
    PATH_TRAVERSAL = "path_traversal"
    XXE = "xxe"
    BRUTE_FORCE = "brute_force"
    JSONP = "jsonp"
    SSRF = "ssrf"
    BASELINE = "baseline"
    REDIRECT = "redirect"
    CRLF = "crlf"
    SENSITIVE = "sensitive"
    SMUGGLING = 'smuggling'
